using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Keyoti.SearchEngine;
using System.Collections;
using System.Text.RegularExpressions;

namespace Plugin
{
    public class ExtendedHtmlDocumentParser : Keyoti.SearchEngine.Documents.HtmlDocumentParser
    {
        public static string keywordStartMark = "@#$", keywordEndMark = "$#@", descStartMark = "$%^", descEndMark = "^%$";
        public ExtendedHtmlDocumentParser(Configuration c) : base(c) { }

        public override object[] ReadText(System.IO.Stream stream, Uri uri, Encoding encoding)
        {
            //To read the meta tags, we need a copy of the document.
            MemoryStream peakStream = CopyStream(stream);
            StreamReader peakReader = new StreamReader(peakStream);
            string documentContent = peakReader.ReadToEnd();
            peakReader.Close();

            //Now read the meta tags
            Hashtable _metaTable = ReadMetaTags(documentContent);
           
            //Add meta contents to a new stream
            MemoryStream modifiedStream = new MemoryStream();
            StreamWriter modWriter = new StreamWriter(modifiedStream);
            modWriter.Write(documentContent + " ");
            if (_metaTable["keywords"] != null)
            {
                modWriter.Write(keywordStartMark);
                modWriter.Write(_metaTable["keywords"].ToString());
                modWriter.Write(keywordEndMark);

                Keyoti.SearchEngine.DataAccess.Log.WriteLogEntry("Plug-in", "Meta keywords:" + _metaTable["keywords"].ToString(), Configuration);
            }
            if (_metaTable["description"] != null)
            {
                modWriter.Write(descStartMark);
                modWriter.Write(_metaTable["description"].ToString());
                modWriter.Write(descEndMark);

                Keyoti.SearchEngine.DataAccess.Log.WriteLogEntry("Plug-in", "Meta description:" + _metaTable["description"].ToString(), Configuration);
            }

            
            

            //Reset the stream
            modWriter.Flush();
            modifiedStream.Position = 0;

            //And do the usual parsing based on the new stream
            return base.ReadText(modifiedStream, uri, encoding);
        }

        MemoryStream CopyStream(Stream stream)
        {
            MemoryStream memStream = new MemoryStream(8192);
            int c;
            byte[] buf = new byte[4096];
            while ((c = stream.Read(buf, 0, buf.Length)) > 0)
            {
                memStream.Write(buf, 0, c);
            }

            memStream.Position = 0;
            stream.Close();

            return memStream;
        }
        
        Hashtable ReadMetaTags(string documentBody)
        {

            Hashtable _metaTable = new Hashtable();
            Regex metaReg = new Regex("<\\s*meta[^>]*name=[\"']?([^\"'\\s]*)[^>]*content=[\"']?([^\"']*)", RegexOptions.IgnoreCase);
            MatchCollection matches = metaReg.Matches(documentBody);
            if (matches != null)
            {
                foreach (Match m in matches)
                {
                    Group name = m.Groups[1];
                    Group content = m.Groups[2];
                    if (name != null && content != null && !_metaTable.ContainsKey(name.Value.ToLower()))
                        _metaTable.Add(name.Value.ToLower(), content.Value);
                }
            }
            return _metaTable;
        }

    }
}
